@@ -56,6 +56,7 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.concurrent.locks.ReentrantLock;
 
 import org.apache.hc.core5.http.ConnectionReuseStrategy;
 import org.apache.hc.core5.http.ContentLengthStrategy;
@@ -100,6 +101,7 @@ import org.apache.hc.core5.http.nio.NHttpMessageWriter;
 import org.apache.hc.core5.http.nio.ResponseChannel;
 import org.apache.hc.core5.http.nio.SessionOutputBuffer;
 import org.apache.hc.core5.http.nio.entity.AsyncEntityProducers;
+import org.apache.hc.core5.http.nio.entity.BasicAsyncEntityProducer;
 import org.apache.hc.core5.http.nio.entity.DigestingEntityConsumer;
 import org.apache.hc.core5.http.nio.entity.DigestingEntityProducer;
 import org.apache.hc.core5.http.nio.entity.StringAsyncEntityConsumer;
@@ -122,10 +124,11 @@ import org.apache.hc.core5.http.protocol.RequestConnControl;
 import org.apache.hc.core5.http.protocol.RequestContent;
 import org.apache.hc.core5.http.protocol.RequestTargetHost;
 import org.apache.hc.core5.http.protocol.RequestValidateHost;
+import org.apache.hc.core5.http.support.BasicResponseBuilder;
 import org.apache.hc.core5.reactor.IOSession;
 import org.apache.hc.core5.reactor.ProtocolIOSession;
 import org.apache.hc.core5.testing.SSLTestContexts;
-import org.apache.hc.core5.testing.nio.extension.Http1TestResources;
+import org.apache.hc.core5.testing.extension.nio.Http1TestResources;
 import org.apache.hc.core5.util.CharArrayBuffer;
 import org.apache.hc.core5.util.TextUtils;
 import org.apache.hc.core5.util.Timeout;
@@ -134,13 +137,15 @@ import org.junit.jupiter.api.Assertions;
 import org.junit.jupiter.api.Test;
 import org.junit.jupiter.api.extension.RegisterExtension;
 
-public abstract class Http1IntegrationTest {
+abstract class Http1IntegrationTest {
 
     private static final Timeout TIMEOUT = Timeout.ofMinutes(1);
     private static final Timeout LONG_TIMEOUT = Timeout.ofMinutes(2);
 
     private final URIScheme scheme;
 
+    private final ReentrantLock lock = new ReentrantLock();
+
     @RegisterExtension
     private final Http1TestResources resources;
 
@@ -158,7 +163,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testSimpleGet() throws Exception {
+    void testSimpleGet() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -185,7 +190,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testSimpleGetConnectionClose() throws Exception {
+    void testSimpleGetConnectionClose() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -215,13 +220,13 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testSimpleGetIdentityTransfer() throws Exception {
+    void testSimpleGetIdentityTransfer() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
         server.register("/hello", () -> new SingleLineResponseHandler("Hi there"));
-        final HttpProcessor httpProcessor = new DefaultHttpProcessor(new RequestValidateHost());
-        final InetSocketAddress serverEndpoint = server.start(httpProcessor, Http1Config.DEFAULT);
+        server.configure(new DefaultHttpProcessor(new RequestValidateHost()));
+        final InetSocketAddress serverEndpoint = server.start();
 
         client.start();
 
@@ -249,13 +254,13 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testPostIdentityTransfer() throws Exception {
+    void testPostIdentityTransfer() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
         server.register("/hello", () -> new SingleLineResponseHandler("Hi there"));
-        final HttpProcessor httpProcessor = new DefaultHttpProcessor(new RequestValidateHost());
-        final InetSocketAddress serverEndpoint = server.start(httpProcessor, Http1Config.DEFAULT);
+        server.configure(new DefaultHttpProcessor(new RequestValidateHost()));
+        final InetSocketAddress serverEndpoint = server.start();
 
         client.start();
 
@@ -284,13 +289,13 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testPostIdentityTransferOutOfSequenceResponse() throws Exception {
+    void testPostIdentityTransferOutOfSequenceResponseNotOK() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
         server.register("/hello", () -> new ImmediateResponseExchangeHandler(500, "Go away"));
-        final HttpProcessor httpProcessor = new DefaultHttpProcessor(new RequestValidateHost());
-        final InetSocketAddress serverEndpoint = server.start(httpProcessor, Http1Config.DEFAULT);
+        server.configure(new DefaultHttpProcessor(new RequestValidateHost()));
+        final InetSocketAddress serverEndpoint = server.start();
 
         client.start();
 
@@ -319,7 +324,77 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testSimpleGetsPipelined() throws Exception {
+    void testPostOutOfSequenceResponseOK() throws Exception {
+        final Http1TestServer server = resources.server();
+        final Http1TestClient client = resources.client();
+
+        server.register("/hello", () -> new ImmediateResponseExchangeHandler(200, "Welcome"));
+        final InetSocketAddress serverEndpoint = server.start();
+
+        client.start();
+
+        final int reqNo = 5;
+
+        final Future<ClientSessionEndpoint> connectFuture = client.connect("localhost", serverEndpoint.getPort(), TIMEOUT);
+        final ClientSessionEndpoint streamEndpoint = connectFuture.get();
+
+        for (int i = 0; i < reqNo; i++) {
+            final Future<Message<HttpResponse, String>> future = streamEndpoint.execute(
+                    new BasicRequestProducer(Method.POST,
+                            createRequestURI(serverEndpoint, "/hello"),
+                            new MultiLineEntityProducer("Hello", 512 * i)),
+                    new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
+            final Message<HttpResponse, String> result = future.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
+
+            Assertions.assertNotNull(result);
+            final HttpResponse response = result.getHead();
+            final String entity = result.getBody();
+            Assertions.assertNotNull(response);
+            Assertions.assertEquals(200, response.getCode());
+            Assertions.assertEquals("Welcome", entity);
+        }
+    }
+
+    @Test
+    void testPostOutOfSequenceResponseOKConnectionClose() throws Exception {
+        final Http1TestServer server = resources.server();
+        final Http1TestClient client = resources.client();
+
+        server.register("/hello", () -> new ImmediateResponseExchangeHandler(
+                BasicResponseBuilder.create(200)
+                        .addHeader(HttpHeaders.CONNECTION, "Close")
+                        .build(),
+                "Welcome"));
+        final InetSocketAddress serverEndpoint = server.start();
+
+        client.start();
+
+        final int reqNo = 5;
+
+        for (int i = 0; i < reqNo; i++) {
+            final Future<ClientSessionEndpoint> connectFuture = client.connect("localhost", serverEndpoint.getPort(), TIMEOUT);
+            final ClientSessionEndpoint streamEndpoint = connectFuture.get();
+
+            final Future<Message<HttpResponse, String>> future = streamEndpoint.execute(
+                    new BasicRequestProducer(Method.POST,
+                            createRequestURI(serverEndpoint, "/hello"),
+                            new MultiLineEntityProducer("Hello", 512 * i)),
+                    new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
+            final Message<HttpResponse, String> result = future.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
+
+            streamEndpoint.close();
+
+            Assertions.assertNotNull(result);
+            final HttpResponse response = result.getHead();
+            final String entity = result.getBody();
+            Assertions.assertNotNull(response);
+            Assertions.assertEquals(200, response.getCode());
+            Assertions.assertEquals("Welcome", entity);
+        }
+    }
+
+    @Test
+    void testSimpleGetsPipelined() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -350,7 +425,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testLargeGet() throws Exception {
+    void testLargeGet() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -396,7 +471,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testLargeGetsPipelined() throws Exception {
+    void testLargeGetsPipelined() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -431,7 +506,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testBasicPost() throws Exception {
+    void testBasicPost() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -459,7 +534,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testBasicPostPipelined() throws Exception {
+    void testBasicPostPipelined() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -491,7 +566,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testHttp10Post() throws Exception {
+    void testHttp10Post() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -520,7 +595,29 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testNoEntityPost() throws Exception {
+    void testHTTP11FeaturesDisabledWithHTTP10Requests() throws Exception {
+        final Http1TestServer server = resources.server();
+        final Http1TestClient client = resources.client();
+
+        server.register("/hello", () -> new SingleLineResponseHandler("Hi back"));
+        final InetSocketAddress serverEndpoint = server.start();
+
+        client.start();
+        final Future<ClientSessionEndpoint> connectFuture = client.connect(
+                "localhost", serverEndpoint.getPort(), TIMEOUT);
+        final ClientSessionEndpoint streamEndpoint = connectFuture.get();
+
+        final HttpRequest request = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/hello"));
+        request.setVersion(HttpVersion.HTTP_1_0);
+        final Future<Message<HttpResponse, String>> future = streamEndpoint.execute(
+                new BasicRequestProducer(request, new BasicAsyncEntityProducer(new byte[] {'a', 'b', 'c'}, null, true)),
+                new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
+        final ExecutionException exception = Assertions.assertThrows(ExecutionException.class, future::get);
+        Assertions.assertInstanceOf(ProtocolException.class, exception.getCause());
+    }
+
+    @Test
+    void testNoEntityPost() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -548,7 +645,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testLargePost() throws Exception {
+    void testLargePost() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -580,7 +677,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testPostsPipelinedLargeResponse() throws Exception {
+    void testPostsPipelinedLargeResponse() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -617,7 +714,7 @@ public abstract class Http1IntegrationTest {
 
 
     @Test
-    public void testLargePostsPipelined() throws Exception {
+    void testLargePostsPipelined() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -653,7 +750,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testSimpleHead() throws Exception {
+    void testSimpleHead() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -679,7 +776,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testSimpleHeadConnectionClose() throws Exception {
+    void testSimpleHeadConnectionClose() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -708,7 +805,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testHeadPipelined() throws Exception {
+    void testHeadPipelined() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -738,7 +835,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testExpectationFailed() throws Exception {
+    void testExpectationFailed() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -753,7 +850,7 @@ public abstract class Http1IntegrationTest {
 
             }
         });
-        final InetSocketAddress serverEndpoint = server.start(null, handler -> new BasicAsyncServerExpectationDecorator(handler) {
+        server.configure(handler -> new BasicAsyncServerExpectationDecorator(handler) {
 
             @Override
             protected AsyncResponseProducer verify(final HttpRequest request, final HttpContext context) throws IOException, HttpException {
@@ -764,71 +861,73 @@ public abstract class Http1IntegrationTest {
                     return new BasicResponseProducer(HttpStatus.SC_UNAUTHORIZED, "You shall not pass");
                 }
             }
-        }, Http1Config.DEFAULT);
+        });
+        final InetSocketAddress serverEndpoint = server.start();
 
         client.start();
         final Future<IOSession> sessionFuture = client.requestSession(
                 new HttpHost("localhost", serverEndpoint.getPort()), TIMEOUT, null);
         final IOSession ioSession = sessionFuture.get();
-        final ClientSessionEndpoint streamEndpoint = new ClientSessionEndpoint(ioSession);
+        try (final ClientSessionEndpoint streamEndpoint = new ClientSessionEndpoint(ioSession)) {
 
-        final HttpRequest request1 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
-        request1.addHeader("password", "secret");
-        final Future<Message<HttpResponse, String>> future1 = streamEndpoint.execute(
-                new BasicRequestProducer(request1, new MultiLineEntityProducer("0123456789abcdef", 1000)),
-                new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
-        final Message<HttpResponse, String> result1 = future1.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
-        Assertions.assertNotNull(result1);
-        final HttpResponse response1 = result1.getHead();
-        Assertions.assertNotNull(response1);
-        Assertions.assertEquals(200, response1.getCode());
-        Assertions.assertEquals("All is well", result1.getBody());
+            final HttpRequest request1 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
+            request1.addHeader("password", "secret");
+            final Future<Message<HttpResponse, String>> future1 = streamEndpoint.execute(
+                    new BasicRequestProducer(request1, new MultiLineEntityProducer("0123456789abcdef", 1000)),
+                    new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
+            final Message<HttpResponse, String> result1 = future1.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
+            Assertions.assertNotNull(result1);
+            final HttpResponse response1 = result1.getHead();
+            Assertions.assertNotNull(response1);
+            Assertions.assertEquals(200, response1.getCode());
+            Assertions.assertEquals("All is well", result1.getBody());
 
-        Assertions.assertTrue(ioSession.isOpen());
+            Assertions.assertTrue(ioSession.isOpen());
 
-        final HttpRequest request2 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
-        final Future<Message<HttpResponse, String>> future2 = streamEndpoint.execute(
-                new BasicRequestProducer(request2, new MultiLineEntityProducer("0123456789abcdef", 5000)),
-                new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
-        final Message<HttpResponse, String> result2 = future2.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
-        Assertions.assertNotNull(result2);
-        final HttpResponse response2 = result2.getHead();
-        Assertions.assertNotNull(response2);
-        Assertions.assertEquals(HttpStatus.SC_UNAUTHORIZED, response2.getCode());
-        Assertions.assertEquals("You shall not pass", result2.getBody());
+            final HttpRequest request2 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
+            final Future<Message<HttpResponse, String>> future2 = streamEndpoint.execute(
+                    new BasicRequestProducer(request2, new MultiLineEntityProducer("0123456789abcdef", 5000)),
+                    new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
+            final Message<HttpResponse, String> result2 = future2.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
+            Assertions.assertNotNull(result2);
+            final HttpResponse response2 = result2.getHead();
+            Assertions.assertNotNull(response2);
+            Assertions.assertEquals(HttpStatus.SC_UNAUTHORIZED, response2.getCode());
+            Assertions.assertEquals("You shall not pass", result2.getBody());
 
-        Assertions.assertTrue(ioSession.isOpen());
+            Assertions.assertTrue(ioSession.isOpen());
 
-        final HttpRequest request3 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
-        request3.addHeader("password", "secret");
-        final Future<Message<HttpResponse, String>> future3 = streamEndpoint.execute(
-                new BasicRequestProducer(request3, new MultiLineEntityProducer("0123456789abcdef", 1000)),
-                new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
-        final Message<HttpResponse, String> result3 = future3.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
-        Assertions.assertNotNull(result3);
-        final HttpResponse response3 = result3.getHead();
-        Assertions.assertNotNull(response3);
-        Assertions.assertEquals(200, response3.getCode());
-        Assertions.assertEquals("All is well", result3.getBody());
+            final HttpRequest request3 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
+            request3.addHeader("password", "secret");
+            final Future<Message<HttpResponse, String>> future3 = streamEndpoint.execute(
+                    new BasicRequestProducer(request3, new MultiLineEntityProducer("0123456789abcdef", 1000)),
+                    new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
+            final Message<HttpResponse, String> result3 = future3.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
+            Assertions.assertNotNull(result3);
+            final HttpResponse response3 = result3.getHead();
+            Assertions.assertNotNull(response3);
+            Assertions.assertEquals(200, response3.getCode());
+            Assertions.assertEquals("All is well", result3.getBody());
 
-        Assertions.assertTrue(ioSession.isOpen());
+            Assertions.assertTrue(ioSession.isOpen());
 
-        final HttpRequest request4 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
-        final Future<Message<HttpResponse, String>> future4 = streamEndpoint.execute(
-                new BasicRequestProducer(request4, AsyncEntityProducers.create("blah")),
-                new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
-        final Message<HttpResponse, String> result4 = future4.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
-        Assertions.assertNotNull(result4);
-        final HttpResponse response4 = result4.getHead();
-        Assertions.assertNotNull(response4);
-        Assertions.assertEquals(HttpStatus.SC_UNAUTHORIZED, response4.getCode());
-        Assertions.assertEquals("You shall not pass", result4.getBody());
+            final HttpRequest request4 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
+            final Future<Message<HttpResponse, String>> future4 = streamEndpoint.execute(
+                    new BasicRequestProducer(request4, AsyncEntityProducers.create("blah")),
+                    new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
+            final Message<HttpResponse, String> result4 = future4.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
+            Assertions.assertNotNull(result4);
+            final HttpResponse response4 = result4.getHead();
+            Assertions.assertNotNull(response4);
+            Assertions.assertEquals(HttpStatus.SC_UNAUTHORIZED, response4.getCode());
+            Assertions.assertEquals("You shall not pass", result4.getBody());
 
-        Assertions.assertFalse(ioSession.isOpen());
+            Assertions.assertFalse(ioSession.isOpen());
+        }
     }
 
     @Test
-    public void testExpectationFailedCloseConnection() throws Exception {
+    void testExpectationFailedCloseConnection() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -843,7 +942,7 @@ public abstract class Http1IntegrationTest {
 
             }
         });
-        final InetSocketAddress serverEndpoint = server.start(null, handler -> new BasicAsyncServerExpectationDecorator(handler) {
+        server.configure(handler -> new BasicAsyncServerExpectationDecorator(handler) {
 
             @Override
             protected AsyncResponseProducer verify(final HttpRequest request, final HttpContext context) throws IOException, HttpException {
@@ -856,33 +955,35 @@ public abstract class Http1IntegrationTest {
                     return new BasicResponseProducer(response, "You shall not pass");
                 }
             }
-        }, Http1Config.DEFAULT);
+        });
+        final InetSocketAddress serverEndpoint = server.start();
 
         client.start();
         final Future<IOSession> sessionFuture = client.requestSession(
                 new HttpHost("localhost", serverEndpoint.getPort()), TIMEOUT, null);
         final IOSession ioSession = sessionFuture.get();
-        final ClientSessionEndpoint streamEndpoint = new ClientSessionEndpoint(ioSession);
+        try (final ClientSessionEndpoint streamEndpoint = new ClientSessionEndpoint(ioSession)) {
 
-        final HttpRequest request1 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
-        final Future<Message<HttpResponse, String>> future1 = streamEndpoint.execute(
-                new BasicRequestProducer(request1, new MultiBinEntityProducer(
-                        new byte[] {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'},
-                        100000,
-                        ContentType.TEXT_PLAIN)),
-                new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
-        final Message<HttpResponse, String> result1 = future1.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
-        Assertions.assertNotNull(result1);
-        final HttpResponse response1 = result1.getHead();
-        Assertions.assertNotNull(response1);
-        Assertions.assertEquals(HttpStatus.SC_UNAUTHORIZED, response1.getCode());
-        Assertions.assertNotNull("You shall not pass", result1.getBody());
+            final HttpRequest request1 = new BasicHttpRequest(Method.POST, createRequestURI(serverEndpoint, "/echo"));
+            final Future<Message<HttpResponse, String>> future1 = streamEndpoint.execute(
+                    new BasicRequestProducer(request1, new MultiBinEntityProducer(
+                            new byte[] {'0', '1', '2', '3', '4', '5', '6', '7', '8', '9', 'a', 'b', 'c', 'd', 'e', 'f'},
+                            100000,
+                            ContentType.TEXT_PLAIN)),
+                    new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
+            final Message<HttpResponse, String> result1 = future1.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
+            Assertions.assertNotNull(result1);
+            final HttpResponse response1 = result1.getHead();
+            Assertions.assertNotNull(response1);
+            Assertions.assertEquals(HttpStatus.SC_UNAUTHORIZED, response1.getCode());
+            Assertions.assertNotNull("You shall not pass", result1.getBody());
 
-        Assertions.assertFalse(streamEndpoint.isOpen());
+            Assertions.assertFalse(streamEndpoint.isOpen());
+        }
     }
 
     @Test
-    public void testDelayedExpectationVerification() throws Exception {
+    void testDelayedExpectationVerification() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -908,8 +1009,11 @@ public abstract class Http1IntegrationTest {
                                 responseChannel.sendInformation(new BasicHttpResponse(HttpStatus.SC_CONTINUE), context);
                             }
                             final HttpResponse response = new BasicHttpResponse(200);
-                            synchronized (entityProducer) {
+                            lock.lock();
+                            try {
                                 responseChannel.sendResponse(response, entityProducer, context);
+                            } finally {
+                                lock.unlock();
                             }
                         }
                     } catch (final Exception ignore) {
@@ -934,15 +1038,21 @@ public abstract class Http1IntegrationTest {
 
             @Override
             public int available() {
-                synchronized (entityProducer) {
+                lock.lock();
+                try {
                     return entityProducer.available();
+                } finally {
+                    lock.unlock();
                 }
             }
 
             @Override
             public void produce(final DataStreamChannel channel) throws IOException {
-                synchronized (entityProducer) {
+                lock.lock();
+                try {
                     entityProducer.produce(channel);
+                } finally {
+                    lock.unlock();
                 }
             }
 
@@ -957,7 +1067,11 @@ public abstract class Http1IntegrationTest {
         });
         final InetSocketAddress serverEndpoint = server.start();
 
-        client.start(Http1Config.custom().setWaitForContinueTimeout(Timeout.ofMilliseconds(100)).build());
+        client.configure(Http1Config.custom()
+                .setWaitForContinueTimeout(Timeout.ofMilliseconds(100))
+                .build());
+        client.start();
+
         final Future<ClientSessionEndpoint> connectFuture = client.connect(
                 "localhost", serverEndpoint.getPort(), TIMEOUT);
         final ClientSessionEndpoint streamEndpoint = connectFuture.get();
@@ -981,7 +1095,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testPrematureResponse() throws Exception {
+    void testPrematureResponse() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1076,14 +1190,17 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testSlowResponseConsumer() throws Exception {
+    void testSlowResponseConsumer() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
         server.register("/", () -> new MultiLineResponseHandler("0123456789abcd", 100));
         final InetSocketAddress serverEndpoint = server.start();
 
-        client.start(Http1Config.custom().setBufferSize(256).build());
+        client.configure(Http1Config.custom()
+                .setBufferSize(256)
+                .build());
+        client.start();
 
         final Future<ClientSessionEndpoint> connectFuture = client.connect(
                 "localhost", serverEndpoint.getPort(), TIMEOUT);
@@ -1130,7 +1247,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testSlowRequestProducer() throws Exception {
+    void testSlowRequestProducer() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1179,7 +1296,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testSlowResponseProducer() throws Exception {
+    void testSlowResponseProducer() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1232,7 +1349,10 @@ public abstract class Http1IntegrationTest {
         });
         final InetSocketAddress serverEndpoint = server.start();
 
-        client.start(Http1Config.custom().setBufferSize(256).build());
+        client.configure(Http1Config.custom()
+                .setBufferSize(256)
+                .build());
+        client.start();
 
         final Future<ClientSessionEndpoint> connectFuture = client.connect(
                 "localhost", serverEndpoint.getPort(), TIMEOUT);
@@ -1256,7 +1376,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testPipelinedConnectionClose() throws Exception {
+    void testPipelinedConnectionClose() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1317,7 +1437,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testPipelinedInvalidRequest() throws Exception {
+    void testPipelinedInvalidRequest() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1414,7 +1534,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testTruncatedChunk() throws Exception {
+    void testTruncatedChunk() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1499,7 +1619,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testExceptionInHandler() throws Exception {
+    void testExceptionInHandler() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1533,10 +1653,11 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testNoServiceHandler() throws Exception {
+    void testNoServiceHandler() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
+        server.register("/ehh", () -> new SingleLineResponseHandler("Hi there"));
         final InetSocketAddress serverEndpoint = server.start();
 
         client.start();
@@ -1557,7 +1678,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testResponseNoContent() throws Exception {
+    void testResponseNoContent() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1591,46 +1712,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testAbsentHostHeader() throws Exception {
-        final Http1TestServer server = resources.server();
-        final Http1TestClient client = resources.client();
-
-        server.register("/hello", () -> new SingleLineResponseHandler("Hi there"));
-        final InetSocketAddress serverEndpoint = server.start();
-
-        client.start(new DefaultHttpProcessor(RequestContent.INSTANCE, RequestConnControl.INSTANCE), Http1Config.DEFAULT);
-
-        final Future<ClientSessionEndpoint> connectFuture = client.connect(
-                "localhost", serverEndpoint.getPort(), TIMEOUT);
-        final ClientSessionEndpoint streamEndpoint = connectFuture.get();
-
-        final HttpRequest request1 = new BasicHttpRequest(Method.GET, createRequestURI(serverEndpoint, "/hello"));
-        request1.setVersion(HttpVersion.HTTP_1_0);
-        final Future<Message<HttpResponse, String>> future1 = streamEndpoint.execute(
-                new BasicRequestProducer(request1, null),
-                new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
-        final Message<HttpResponse, String> result1 = future1.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
-        Assertions.assertNotNull(result1);
-        final HttpResponse response1 = result1.getHead();
-        Assertions.assertNotNull(response1);
-        Assertions.assertEquals(200, response1.getCode());
-        Assertions.assertEquals("Hi there", result1.getBody());
-
-        final HttpRequest request2 = new BasicHttpRequest(Method.GET, createRequestURI(serverEndpoint, "/hello"));
-        request2.setVersion(HttpVersion.HTTP_1_1);
-        final Future<Message<HttpResponse, String>> future2 = streamEndpoint.execute(
-                new BasicRequestProducer(request2, null),
-                new BasicResponseConsumer<>(new StringAsyncEntityConsumer()), null);
-        final Message<HttpResponse, String> result2 = future2.get(TIMEOUT.getDuration(), TIMEOUT.getTimeUnit());
-        Assertions.assertNotNull(result2);
-        final HttpResponse response2 = result2.getHead();
-        Assertions.assertNotNull(response2);
-        Assertions.assertEquals(400, response2.getCode());
-        Assertions.assertEquals("Host header is absent", result2.getBody());
-    }
-
-    @Test
-    public void testMessageWithTrailers() throws Exception {
+    void testMessageWithTrailers() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1688,7 +1770,7 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testProtocolException() throws Exception {
+    void testProtocolException() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
@@ -1764,14 +1846,15 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testHeaderTooLarge() throws Exception {
+    void testHeaderTooLarge() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
         server.register("/hello", () -> new SingleLineResponseHandler("Hi there"));
-        final InetSocketAddress serverEndpoint = server.start(null, Http1Config.custom()
+        server.configure(Http1Config.custom()
                 .setMaxLineLength(100)
                 .build());
+        final InetSocketAddress serverEndpoint = server.start();
         client.start();
 
         final Future<ClientSessionEndpoint> connectFuture = client.connect(
@@ -1793,16 +1876,18 @@ public abstract class Http1IntegrationTest {
     }
 
     @Test
-    public void testHeaderTooLargePost() throws Exception {
+    void testHeaderTooLargePost() throws Exception {
         final Http1TestServer server = resources.server();
         final Http1TestClient client = resources.client();
 
         server.register("/hello", () -> new SingleLineResponseHandler("Hi there"));
-        final InetSocketAddress serverEndpoint = server.start(null, Http1Config.custom()
+        server.configure(Http1Config.custom()
                 .setMaxLineLength(100)
                 .build());
-        client.start(
-                new DefaultHttpProcessor(RequestContent.INSTANCE, RequestTargetHost.INSTANCE, RequestConnControl.INSTANCE), null);
+        final InetSocketAddress serverEndpoint = server.start();
+        client.configure(
+                new DefaultHttpProcessor(RequestContent.INSTANCE, RequestTargetHost.INSTANCE, RequestConnControl.INSTANCE));
+        client.start();
 
         final Future<ClientSessionEndpoint> connectFuture = client.connect(
                 "localhost", serverEndpoint.getPort(), TIMEOUT);
